@@ -1,5 +1,5 @@
 /*
- cdrfifo.c , Copyright 2006 Thomas Schmitt <scdbackup@gmx.net>
+ cdrfifo.c , Copyright 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
 
  A fd-to-fd or fd-to-memory fifo to be used within cdrskin or independently.
  By chaining of fifo objects, several fifos can be run simultaneously
@@ -25,11 +25,16 @@
 #include <sys/time.h>
 #include <sys/select.h>
 
+#ifndef Cdrfifo_standalonE
+/* <<< until release of 0.7.4 : for Libburn_has_open_trac_srC */
+#include "../libburn/libburn.h"
+#endif
+
 #include "cdrfifo.h"
 
 
 /* Macro for creation of arrays of objects (or single objects) */
-#define TSOB_FELD(typ,anz) (typ *) malloc((anz)*sizeof(typ));
+#define TSOB_FELD(typ,anz) (typ *) calloc(anz, sizeof(typ));
 
 
 #define Cdrfifo_buffer_chunK 2048
@@ -120,7 +125,7 @@ struct CdrfifO {
                    struct burn_source object.
     @param chunk_size Size of buffer block for a single transaction (0=default)
     @param buffer_size Size of fifo buffer
-    @param flag Unused yet
+    @param flag bit0= Debugging verbosity
     @return 1 on success, <=0 on failure
 */
 int Cdrfifo_new(struct CdrfifO **ff, int source_fd, int dest_fd,
@@ -174,7 +179,13 @@ int Cdrfifo_new(struct CdrfifO **ff, int source_fd, int dest_fd,
  o->follow_up_fd_idx= -1;
  o->next= o->prev= NULL;
  o->chain_idx= 0;
+
+#ifdef Libburn_has_open_trac_srC
+ o->buffer= burn_os_alloc_buffer((size_t) buffer_size, 0);
+#else
  o->buffer= TSOB_FELD(char,buffer_size);
+#endif /* ! Libburn_has_open_trac_srC */
+
  if(o->buffer==NULL)
    goto failed;
  return(1);
@@ -192,10 +203,11 @@ int Cdrfifo_close(struct CdrfifO *o, int flag)
  if(o->dest_fd!=-1)
    close(o->dest_fd);
  o->dest_fd= -1;
- for(i=0; i<o->follow_up_fd_counter; i++)
+ for(i=0; i<o->follow_up_fd_counter; i++) {
    if(o->follow_up_fds[i][1]!=-1)
      close(o->follow_up_fds[i][1]);
    o->follow_up_fds[i][1]= -1;
+ }
  return(1);
 }
 
@@ -226,8 +238,14 @@ int Cdrfifo_destroy(struct CdrfifO **ff, int flag)
 
  if(o->iso_fs_descr!=NULL)
    free((char *) o->iso_fs_descr);
+
  if(o->buffer!=NULL)
+#ifdef Libburn_has_open_trac_srC
+   burn_os_free_buffer(o->buffer, o->buffer_size, 0);
+#else
    free((char *) o->buffer);
+#endif /* Libburn_has_open_trac_srC */
+
  free((char *) o);
  (*ff)= NULL;
  return(1);
@@ -616,7 +634,7 @@ return: <0 = error , 0 = idle , 1 = did some work
 */
 {
  double buffer_space;
- int can_read,can_write,ret,did_work= 0,idx,sod,eop_is_near,eop_idx;
+ int can_read,can_write= 0,ret,did_work= 0,idx,sod, eop_idx;
 
  buffer_space= Cdrfifo_tell_buffer_space(o,0);
  if(o->dest_fd>=0) if(FD_ISSET((o->dest_fd),wts)) {
@@ -626,7 +644,7 @@ return: <0 = error , 0 = idle , 1 = did some work
    if(o->read_idx+can_write > o->buffer_size)
      can_write= o->buffer_size - o->read_idx;
    if(o->follow_up_fd_idx>=0) {
-     eop_is_near= Cdrfifo_eop_adjust(o,&can_write,&eop_idx,0);
+     Cdrfifo_eop_adjust(o,&can_write,&eop_idx,0);
      if(can_write<=0)
        goto after_write;
    }
@@ -659,6 +677,46 @@ return: <0 = error , 0 = idle , 1 = did some work
 after_write:;
  if(o->source_fd>=0) if(FD_ISSET((o->source_fd),rds)) {
    can_read= o->buffer_size - o->write_idx;
+
+#ifdef Libburn_has_open_trac_srC
+
+   /* ts A91115
+      This chunksize must be aligned to filesystem blocksize.
+    */
+#define Cdrfifo_o_direct_chunK 32768
+
+   if(o->write_idx < o->read_idx && o->write_idx + can_read > o->read_idx)
+     can_read= o->read_idx - o->write_idx;
+   if(o->fd_in_limit>=0.0)
+     if(can_read > o->fd_in_limit - o->fd_in_counter)
+       can_read= o->fd_in_limit - o->fd_in_counter;
+   /* Make sure to read with properly aligned size */
+   if(can_read > Cdrfifo_o_direct_chunK)
+     can_read= Cdrfifo_o_direct_chunK;
+   else if(can_read < Cdrfifo_o_direct_chunK)
+     can_read= -1;
+   ret= 0;
+   if(can_read>0)
+     ret= read(o->source_fd,o->buffer+o->write_idx,can_read);
+   if(can_read < 0) {
+     /* waiting for a full Cdrfifo_o_direct_chunK to fit */
+     if(can_write <= 0 && o->dest_fd >= 0) {
+        fd_set rds,wts,exs;
+        struct timeval wt;
+
+        FD_ZERO(&rds);
+        FD_ZERO(&wts);
+        FD_ZERO(&exs);
+        FD_SET((o->dest_fd),&wts);
+        wt.tv_sec=  0;
+        wt.tv_usec= 10000;
+        select(o->dest_fd + 1,&rds, &wts, &exs, &wt);
+
+     }
+   } else
+
+#else /* Libburn_has_open_trac_srC */
+
    if(can_read>o->chunk_size)
      can_read= o->chunk_size;
    if(o->write_idx<o->read_idx && o->write_idx+can_read > o->read_idx)
@@ -669,6 +727,9 @@ after_write:;
    ret= 0;
    if(can_read>0)
      ret= read(o->source_fd,o->buffer+o->write_idx,can_read);
+
+#endif /* ! Libburn_has_open_trac_srC */
+
    if(ret==-1) {
 
      /* >>> handle input error */;
@@ -738,7 +799,7 @@ ex:;
 /** Check for pending data at the fifo's source file descriptor and wether the
     fifo is ready to take them. Simultaneously check the buffer for existing
     data and the destination fd for readiness to accept some. If so, a small
-    chunk of data is transfered to and/or from the fifo.
+    chunk of data is transferred to and/or from the fifo.
     This is done for the given fifo object and all members of its next-chain.
     The check and transactions are repeated until a given timespan has elapsed.
     libburn applications call this function in the burn loop instead of sleep().
@@ -978,9 +1039,9 @@ int Test_mixed_bs(char **paths, int path_count,
  bit0= debugging verbousity
 */
 {
- int fd_in[100],fd_out[100],ret,pipe_fds[100][2],real_out[100];
+ int fd_in[100],fd_out[100],ret,pipe_fds[100][2];
  int i,iv,stall_counter= 0,cycle_counter= 0.0;
- char buf[10240], target_path[80];
+ char target_path[80];
  double in_counter, out_counter, prev_in= -1.0, prev_out= -1.0;
  struct CdrfifO *ff_in= NULL, *ff_out= NULL;
 
@@ -1048,9 +1109,8 @@ int Test_multi(int fs_size, double speed_limit, double interval, int flag)
  bit0= debugging verbousity
 */
 {
- int fd_in[4],fd_out[4],ret,pipe_fds[4][2],real_out[4],pipe_idx;
+ int fd_in[4],fd_out[4],ret,pipe_fds[4][2];
  int i,iv;
- char buf[10240];
  struct CdrfifO *ff1= NULL,*ff2= NULL;
 
  /* open four pairs of fds */
