@@ -1,5 +1,5 @@
 
-/* Copyright (c) 2011 Thomas Schmitt <scdbackup@gmx.net>
+/* Copyright (c) 2011 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -1085,6 +1085,7 @@ int burn_cdtext_from_packfile(char *path, unsigned char **text_packs,
 
 	BURN_ALLOC_MEM(msg, char, 4096);
 
+	*text_packs = NULL;
 	if (stat(path, &stbuf) == -1) {
 cannot_open:;
 		sprintf(msg, "Cannot open CD-TEXT pack file '%.4000s'", path);
@@ -1136,7 +1137,14 @@ cannot_read:;
 			path);
 		libdax_msgs_submit(libdax_messenger, -1, 0x0002018b,
 				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
-				burn_printify(msg), errno, 0);
+				burn_printify(msg), 0, 0);
+		ret = 0; goto ex;
+	} if (*num_packs <= 0) {
+		strcpy(msg,
+			"CD-Text pack file contains no complete text pack");
+		libdax_msgs_submit(libdax_messenger, -1, 0x000201aa,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				burn_printify(msg), 0, 0);
 		ret = 0; goto ex;
 	}
 
@@ -1251,7 +1259,7 @@ static int collect_payload(unsigned char *text_packs, int num_packs,
 	break;
 		memcpy(*payload + *payload_count, pack + 4, 12);
 		*payload_count += 12;
-		if (pack[4] & 128)
+		if (pack[3] & 128)
 			double_byte = 1;
 	}
 	(*payload)[*payload_count] = 0;
@@ -1612,7 +1620,7 @@ static int burn_make_v07t(unsigned char *text_packs, int num_packs,
 		return 0;
 	}
 
-	/* Obtain first_tno and last_tno from type 0x88 if present. */
+	/* Obtain first_tno and last_tno from type 0x8f if present. */
 	if (first_tno <= 0) {
 		if (pack[5] > 0 &&  pack[5] + pack[6] < 100 &&
 		    pack[5] <= pack[6]) {
@@ -1639,6 +1647,21 @@ static int burn_make_v07t(unsigned char *text_packs, int num_packs,
 	/* Report content */
 	result_size = 0;
 	for (block = 0; block < 8; block++) {
+		/* Obtain character code, reject unknown ones */
+		ret = search_pack(text_packs, num_packs, 0, 0x8f, block, 
+                          	&pack, &pack_no, 0);
+		if (ret > 0)
+			*char_code = pack[4];
+		if (*char_code != 0x00 && *char_code != 0x01 &&
+		    *char_code != 0x80) {
+			sprintf(msg,
+			  "CD-TEXT block %d with unknown character code %2.2x",
+			  block, (unsigned int) *char_code);
+			libdax_msgs_submit(libdax_messenger, -1, 0x0002019f,
+					LIBDAX_MSGS_SEV_FAILURE,
+					LIBDAX_MSGS_PRIO_HIGH, msg, 0, 0);
+			return 0;
+		}
 		ret = report_block(text_packs, num_packs, block, 
 		                   first_tno, last_tno, *char_code,
 		                   result, result_size, flag & 1);
@@ -1648,16 +1671,7 @@ static int burn_make_v07t(unsigned char *text_packs, int num_packs,
 	continue;
 		result_size = ret;
 	}
-
-#ifdef NIX
-	if (flag & 1)
-		return result_size;
-	return (int) strlen((char *) result);
-#else /* NIX */
-
 	return result_size;
-
-#endif /* ! NIX */
 }
 
 
