@@ -128,7 +128,7 @@ int sg_initialize(char msg[1024], int flag)
 
 /* ts A91227 */
 /** Performs global finalization of the SCSI transport adapter and eventually
-    needed operating system facilities. Releases globally aquired resources.
+    needed operating system facilities. Releases globally acquired resources.
     @param flag  unused yet, submit 0
     @return      1 = success, <=0 = failure
 */  
@@ -441,6 +441,11 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	struct burn_drive *t;
 	struct burn_drive out;
 
+	/* Initialize pointers to managed memory */
+	out.devname = NULL;
+	out.idata = NULL;
+	out.mdata = NULL;
+
 	/* ts A60923 */
 	out.bus_no = bus_no;
 	out.host = host_no;
@@ -449,6 +454,8 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.lun = lun_no;
 
 	out.devname = strdup(fname);
+	if (out.devname == NULL)
+		goto could_not_allocate;
 
 	out.cam = NULL;
 	out.lock_fd = -1;
@@ -492,13 +499,23 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.idata = calloc(1, sizeof(struct burn_scsi_inquiry_data));
 	out.idata->valid = 0;
 	out.mdata = calloc(1, sizeof(struct scsi_mode_data));
-	out.mdata->valid = 0;
 	if (out.idata == NULL || out.mdata == NULL) {
+could_not_allocate:;
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020108,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
 			"Could not allocate new drive object", 0, 0);
+		if (out.devname != NULL)
+			free(out.devname);
+		out.devname = NULL;
+		if (out.idata != NULL)
+			free(out.idata);
+		out.idata = NULL;
+		if (out.mdata != NULL)
+			free(out.mdata);
+		out.mdata = NULL;
 		return;
 	}
+	out.mdata->p2a_valid = 0;
 	memset(&out.params, 0, sizeof(struct params));
 	t = burn_drive_register(&out);
 
@@ -978,7 +995,7 @@ ex:;
 
 /* ts B00115 */
 /* Return 1 if the given path leads to a regular file or a device that can be
-   seeked, read and eventually written with 2 kB granularity.
+   fseeked, read and eventually written with 2 kB granularity.
 */
 int burn_os_is_2k_seekrw(char *path, int flag)
 {
@@ -1051,7 +1068,7 @@ int burn_os_is_2k_seekrw(char *path, int flag)
                   0 = could not estimate size capacity of file object
                   1 = estimation has been made, bytes was set
 */
-int burn_os_stdio_capacity(char *path, off_t *bytes)
+int burn_os_stdio_capacity(char *path, off_t write_start, off_t *bytes)
 {
 	struct stat stbuf;
 	struct statvfs vfsbuf;
@@ -1104,7 +1121,7 @@ int burn_os_stdio_capacity(char *path, off_t *bytes)
 			{ret = -2; goto ex;}
 		*bytes = add_size;
 	} else if(S_ISREG(stbuf.st_mode)) {
-		add_size = stbuf.st_blocks * (off_t) 512;
+		add_size = burn_sparse_file_addsize(write_start, &stbuf);
 		strcpy(testpath, path);
 	} else
 		{ret = 0; goto ex;}
