@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2017 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -1419,7 +1419,7 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 	enum burn_write_types wt;
 	struct burn_drive *d = o->drive;
 	char *msg = NULL, *reason_pt;
-	int no_media = 0, ret, has_cdtext;
+	int no_media = 0, ret, has_cdtext, is_bd_pow = 0;
 
 	reason_pt= reasons;
 	reasons[0] = 0;
@@ -1497,6 +1497,20 @@ int burn_precheck_write(struct burn_write_opts *o, struct burn_disc *disc,
 		   sequential stdio "drive" */
 		if (o->start_byte >= 0)
 			strcat(reasons, "write start address not supported, ");
+
+		is_bd_pow = burn_drive_get_bd_r_pow(d);
+		if (is_bd_pow && !silent) 
+			libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x0002011e,
+				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
+			   "Unsuitable media detected: BD-R formatted to POW.",
+				0, 0);
+		if (is_bd_pow) {
+			strcat(reasons,
+				"unsuitable media formatting POW detected, ");
+			return 0;
+		}
+
 	} else {
 unsuitable_profile:;
 		msg = calloc(1, 160);
@@ -2589,6 +2603,8 @@ int burn_stdio_write(int fd, char *buf, int count, struct burn_drive *d,
 
 	if (d->cancel || count <= 0)
 		return 0;
+	if(d->do_simulate)
+		return 1;
 
 	todo = count;
 	done = 0;
@@ -2619,6 +2635,7 @@ fprintf(stderr, "libburn_DEBUG: write(%d, %lX, %d)\n",
 		d->cancel = 1;
 		ret = 0; goto ex;
 	}
+	ret = 1;
 ex:;
 	BURN_FREE_MEM(msg);
 	return ret;
@@ -2794,6 +2811,7 @@ int burn_stdio_write_track(struct burn_write_opts *o, struct burn_session *s,
 		d->write = burn_stdio_mmc_dummy_write;
 	else
 		d->write = burn_stdio_mmc_write;
+	d->do_simulate = o->simulate;
 	d->sync_cache = burn_stdio_mmc_sync_cache;
 
 	burn_stdio_slowdown(d, &prev_time, 0, 1); /* initialize */
@@ -3289,7 +3307,9 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		rpt += d->buffer->bytes;
 		d->buffer->sectors = chunksize;
 		d->nwa = start;
-		if(d->drive_role == 1) {
+		if(d->do_simulate) {
+			err = 0;
+		} else if(d->drive_role == 1) {
 			err = d->write(d, d->nwa, d->buffer);
 		} else {
 			ret = burn_stdio_write(fd, (char *) d->buffer->data,
@@ -3309,7 +3329,9 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 	if(d->drive_role == 1)
 		d->needs_sync_cache = 1;
 	if(flag & 1) {
-		if(d->drive_role == 1)
+		if(d->do_simulate) {
+			;
+		} else if(d->drive_role == 1)
 			d->sync_cache(d);
 		else
 			burn_stdio_sync_cache(fd, d, 2);
